//http://homofaciens.de/technics-physical-computing-set01_en.htm

//Switch a relay after unlocking with RFID RC522 module

// VCC on RC522 to Arduino +5V
// GND on RC522 to Arduino GND
// Reset on RC522 to Arduino pin 9
// SPI SS on RC522 to Arduino pin 10
// SPI MOSI on RC522 to Arduino pin 11
// SPI MISO on RC522 to Arduino pin 12
// SPI SCK on RC522 to Arduino pin 13

//Push button through 1 kiloohm resistor to Arduino pin A3
//Second pin of push button to ground
//Red indicator LED through 220 ohms resistor to Arduino pin A1
//Cathode (short pin) of LED to GND of Arduino
//Relay pin S to Arduino pin A2
//Relay (-) to Arduino GND
//Relay (+) to Arduino +5V


#include <SPI.h>
#include <MFRC522.h>


#define RELAY_PIN           A2
#define ON_BUTTON_PIN       A3
#define CODE_OK_LED_PIN     A0
#define CODE_WRONG_LED_PIN  A1

#define SS_PIN 10
#define RST_PIN 9

#define KEYPAD_ROWS      4
#define KEYPAD_COLS      4

int codeInput[] = {0, 0, 0, 0};
int codeNo = 0;
int codeOK = 0;
unsigned long lastCodeTime = 0;

//MFRC522 mfrc522(SS_PIN, RST_PIN);  // Create MFRC522 instance.
MFRC522::MIFARE_Key key;
// Init array that will store new NUID
byte nuidPICC[4];


// Helper routine to dump a byte array as hex values to Serial.
void printHex(byte *buffer, byte bufferSize) {
  for (byte i = 0; i < bufferSize; i++) {
    Serial.print(buffer[i] < 0x10 ? " 0" : " ");
    Serial.print(buffer[i], HEX);
  }
}

// Helper routine to dump a byte array as dec values to Serial.
void printDec(byte *buffer, byte bufferSize) {
  for (byte i = 0; i < bufferSize; i++) {
    Serial.print(buffer[i] < 0x10 ? " 0" : " ");
    Serial.print(buffer[i], DEC);
  }
}

int checkForRFID(int acceptNew){
  int checkState = 0;

  MFRC522 mfrc522(SS_PIN, RST_PIN);  // Create MFRC522 instance.

  mfrc522.PCD_Init(); // Init MFRC522 card

  // Look for new cards
  if ( ! mfrc522.PICC_IsNewCardPresent())
    return 2;

  // Verify if the NUID has been readed
  if ( ! mfrc522.PICC_ReadCardSerial())
    return 2;


  Serial.print(F("PICC type: "));
  MFRC522::PICC_Type piccType = mfrc522.PICC_GetType(mfrc522.uid.sak);
  Serial.println(mfrc522.PICC_GetTypeName(piccType));

  // Check is the PICC of Classic MIFARE type
  if (piccType != MFRC522::PICC_TYPE_MIFARE_MINI &&
    piccType != MFRC522::PICC_TYPE_MIFARE_1K &&
    piccType != MFRC522::PICC_TYPE_MIFARE_4K) {
    Serial.println(F("Your tag is not of type MIFARE Classic."));
    return;
  }

  if (mfrc522.uid.uidByte[0] != nuidPICC[0] ||
    mfrc522.uid.uidByte[1] != nuidPICC[1] ||
    mfrc522.uid.uidByte[2] != nuidPICC[2] ||
    mfrc522.uid.uidByte[3] != nuidPICC[3] ) {
    if(acceptNew == 1){
      Serial.println(F("A new card has been detected."));

      // Store NUID into nuidPICC array
      for (byte i = 0; i < 4; i++) {
        nuidPICC[i] = mfrc522.uid.uidByte[i];
      }

      Serial.println(F("The NUID tag is:"));
      Serial.print(F("In hex: "));
      printHex(mfrc522.uid.uidByte, mfrc522.uid.size);
      Serial.println();
      Serial.print(F("In dec: "));
      printDec(mfrc522.uid.uidByte, mfrc522.uid.size);
      Serial.println();
    }
  }
  else{
    Serial.println(F("Card read previously."));
    checkState = 1;
  }

  // Halt PICC
  mfrc522.PICC_HaltA();

  // Stop encryption on PCD
  mfrc522.PCD_StopCrypto1();

  return checkState;
}


void setup() {
  pinMode(ON_BUTTON_PIN, INPUT_PULLUP);
  pinMode(RELAY_PIN, OUTPUT);
  pinMode(CODE_OK_LED_PIN, OUTPUT);
  pinMode(CODE_WRONG_LED_PIN, OUTPUT);
  codeNo = 0;
  codeOK = 0;
  lastCodeTime = 0;

  digitalWrite(CODE_WRONG_LED_PIN, HIGH);

  Serial.begin(115200);

  SPI.begin();      // Init SPI bus

  for (byte i = 0; i < 6; i++) {
    key.keyByte[i] = 0xFF;
  }

  Serial.println("Waiting for key to be present.");
  while (checkForRFID(1) != 1);
  Serial.println("RFID key detected");
  printHex(key.keyByte, MFRC522::MF_KEY_SIZE);

  digitalWrite(CODE_WRONG_LED_PIN, LOW);

}


void loop(){

  if(millis() - lastCodeTime > 1000){//Check for RFID each 1000ms = 1s
    if(checkForRFID(0) != 1){
      digitalWrite(CODE_WRONG_LED_PIN, HIGH);
      codeOK = 0;
    }
    else{
      digitalWrite(CODE_WRONG_LED_PIN, LOW);
      codeOK = 1;
    }
    lastCodeTime = millis();
  }
  if(digitalRead(ON_BUTTON_PIN)){
    digitalWrite(RELAY_PIN, LOW);
    delay(100);//Wait for contact bounce to settle
  }
  else{
    if(codeOK == 1){
      digitalWrite(RELAY_PIN, HIGH);
      delay(100);//Wait for contact bounce to settle
    }
  }
}
